package com.nationrel.modules.rp.controller;

import java.util.Arrays;

import com.nationrel.modules.rp.entity.PredictionColumns;
import com.nationrel.modules.rp.service.IPredictionColumnsService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.jeecg.common.api.IPageExtField;
import org.jeecg.common.api.PageExtField;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.config.i18n.MessageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.apache.shiro.authz.annotation.RequiresPermissions;

 /**
 * @Description: prediction_columns
 * @Author: nationrel
 * @Date: 2024-08-26
 * @Version: V1.0
 */
@Tag(name="prediction_columns")
@RestController
@RequestMapping("/rp/predictionColumns")
@Slf4j
public class PredictionColumnsController extends JeecgController<PredictionColumns, IPredictionColumnsService> {
	@Autowired
	private IPredictionColumnsService predictionColumnsService;

	/**
	 * 分页列表查询
	 *
	 * @param predictionColumns
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "prediction_columns-分页列表查询")
	@Operation(summary="prediction_columns-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPageExtField<PredictionColumns>> queryPageList(PredictionColumns predictionColumns,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<PredictionColumns> queryWrapper = QueryGenerator.initQueryWrapper(predictionColumns, req.getParameterMap());
		IPageExtField<PredictionColumns> page = new PageExtField<>(PredictionColumns.class, pageNo, pageSize);
		IPageExtField<PredictionColumns> pageList = predictionColumnsService.page(page, queryWrapper);
		return Result.OK(pageList);
	}

	/**
	 *   添加
	 *
	 * @param predictionColumns
	 * @return
	 */
	@AutoLog(value = "prediction_columns-添加")
	@Operation(summary="prediction_columns-添加")
	@RequiresPermissions("aaa:prediction_columns:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody PredictionColumns predictionColumns) {
		predictionColumnsService.save(predictionColumns);
		return Result.OK(MessageUtils.get("nationrel_add_success"));
	}

	/**
	 *  编辑
	 *
	 * @param predictionColumns
	 * @return
	 */
	@AutoLog(value = "prediction_columns-编辑")
	@Operation(summary="prediction_columns-编辑")
	@RequiresPermissions("aaa:prediction_columns:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody PredictionColumns predictionColumns) {
		predictionColumnsService.updateById(predictionColumns);
		return Result.OK(MessageUtils.get("nationrel_edit_success"));
	}

	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "prediction_columns-通过id删除")
	@Operation(summary="prediction_columns-通过id删除")
	@RequiresPermissions("aaa:prediction_columns:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		predictionColumnsService.removeById(id);
		return Result.OK(MessageUtils.get("nationrel_delete_success"));
	}

	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "prediction_columns-批量删除")
	@Operation(summary="prediction_columns-批量删除")
	@RequiresPermissions("aaa:prediction_columns:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.predictionColumnsService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK(MessageUtils.get("nationrel_delete_batch_success"));
	}

	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "prediction_columns-通过id查询")
	@Operation(summary="prediction_columns-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<PredictionColumns> queryById(@RequestParam(name="id",required=true) String id) {
		PredictionColumns predictionColumns = predictionColumnsService.getById(id);
		if(predictionColumns==null) {
			return Result.error(MessageUtils.get("nationrel_no_found_data"));
		}
		return Result.OK(predictionColumns);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param predictionColumns
    */
    @RequiresPermissions("aaa:prediction_columns:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, PredictionColumns predictionColumns) {
        return super.exportXls(request, predictionColumns, PredictionColumns.class, "prediction_columns");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequiresPermissions("aaa:prediction_columns:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, PredictionColumns.class);
    }

}
