package com.nationrel.modules.rp.service.impl;

import cn.hutool.core.comparator.FuncComparator;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.interfaces.Func;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Maps;
import com.nationrel.modules.rp.constant.RpConstant;
import com.nationrel.modules.rp.dto.PredictionStruTreeImport;
import com.nationrel.modules.rp.entity.PredictionStruTree;
import com.nationrel.modules.rp.mapper.PredictionStruTreeMapper;
import com.nationrel.modules.rp.req.SaveStructureReq;
import com.nationrel.modules.rp.resp.PredictionStruTreeResp;
import com.nationrel.modules.rp.service.IPredictionStruTreeService;
import me.zhyd.oauth.utils.UuidUtils;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.exception.NationrelBootException;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Description: prediction_stru_tree
 * @Author: nationrel
 * @Date:   2024-08-26
 * @Version: V1.0
 */
@Service
public class PredictionStruTreeServiceImpl extends ServiceImpl<PredictionStruTreeMapper, PredictionStruTree> implements IPredictionStruTreeService {

    @Autowired
    private PredictionStruTreeMapper predictionStruTreeMapper;

    @Override
    public List<PredictionStruTreeResp> getLeftTree(Long predictionId, Long struId) {
        int leftNo, rightNo;
        if (ObjectUtils.isEmpty(struId)) {
            PredictionStruTree predictionStruTree = this.getOne(new LambdaQueryWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .eq(PredictionStruTree::getParentStruId, RpConstant.ROOTNODE_PARENT));
            // 取得根节点
            leftNo = predictionStruTree.getLeftNo();
            rightNo = predictionStruTree.getRightNo();
        } else {
            PredictionStruTree predictionStruTree = this.getOne(new LambdaQueryWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .eq(PredictionStruTree::getId, struId));
            leftNo = predictionStruTree.getLeftNo() + 1;
            rightNo = predictionStruTree.getRightNo() - 1;
        }

        List<PredictionStruTree> predictionStruTreeList = this.list(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .between(PredictionStruTree::getLeftNo, leftNo, rightNo));
        List<PredictionStruTreeResp> list = predictionStruTreeList.stream().map(tree -> {
            PredictionStruTreeResp resp = new PredictionStruTreeResp();
            resp.setId(tree.getId());
            resp.setStruId(tree.getStruId());
            resp.setName(tree.getName());
            resp.setRpId(tree.getPredictionId());
            resp.setLeftNo(tree.getLeftNo());
            resp.setRightNo(tree.getRightNo());
            resp.setParentStruId(tree.getParentStruId());
            resp.setStruTypeCode(tree.getStruTypeCode());
            resp.setOpen(true);
            return resp;
        }).sorted(Comparator.comparing(PredictionStruTreeResp::getLeftNo)).toList();
        return list;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addStructure(SaveStructureReq saveStructureReq) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String struId = saveStructureReq.getStruId();
        String predictionId = saveStructureReq.getRpId();
        List<String[]> structures = saveStructureReq.getStructures();
        List<String> newStruList = Lists.newArrayList();
        PredictionStruTree predictionStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getStruId, struId));
        if (ObjectUtils.isEmpty(predictionStruTree)) {
            throw new NationrelBootException("数据不存在");
        }

        PredictionStruTree parentStruNode = null;
        if (predictionStruTree.getParentStruId().equals(RpConstant.ROOTNODE_PARENT) || StringUtils.isEmpty(structures.get(0)[0])) {
            parentStruNode = predictionStruTree;
        } else {
            parentStruNode = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .eq(PredictionStruTree::getStruId, predictionStruTree.getParentStruId()));
        }
        List<PredictionStruTree> struChildrenTreeList = Lists.newArrayList();
        PredictionStruTree rootStrTree = new PredictionStruTree();
        rootStrTree.setPredictionId(predictionId);
        rootStrTree.setStruId(parentStruNode.getStruId());
        rootStrTree.setParentStruId(parentStruNode.getParentStruId());
        rootStrTree.setStruTypeCode(RpConstant.STRU_TYPE_STRU);
        rootStrTree.setName(parentStruNode.getName());
        struChildrenTreeList.add(rootStrTree);
        // 查询当前选中节点最后一个结构子节点，取它的右序号+1作为多级添加的起始左序号
        PredictionStruTree lastChildStruNode = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getParentStruId, parentStruNode.getStruId())
                .orderByDesc(PredictionStruTree::getLeftNo)
                .last("limit 1"));
        Integer rightNo;
        if (lastChildStruNode != null) {
            rightNo = lastChildStruNode.getRightNo() + 1;
        } else {
            rightNo = parentStruNode.getRightNo();
        }


        for (String[] structure : structures) {
            String parentStruId = null;
            for (int i = 0; i < structure.length; i++) {
                String rootStruId = null;

                if (parentStruNode.getParentStruId().equals(RpConstant.ROOTNODE_PARENT)) {
                    rootStruId = parentStruNode.getStruId();
                } else {
                    rootStruId = parentStruNode.getParentStruId();
                }
                String childStruId = UuidUtils.getUUID();
                if (i == 0 && StringUtils.isNotEmpty(structure[i])) {
                    PredictionStruTree struTree = this.buildStruTree(predictionId, childStruId, rootStruId, structure[i]);
                    parentStruId = struTree.getId();
                    struChildrenTreeList.add(struTree);
                    newStruList.add(struTree.getStruId());
                } else {
                    if (StringUtils.isNotEmpty(structure[i])) {
                        if (StringUtils.isEmpty(parentStruId)) {
                            parentStruId = predictionStruTree.getStruId();
                        }
                        PredictionStruTree struTree = this.buildStruTree(predictionId, childStruId, parentStruId, structure[i]);
                        struChildrenTreeList.add(struTree);
                        newStruList.add(struTree.getStruId());
                        parentStruId = childStruId;
                    }
                }

            }
        }
        Map<String, PredictionStruTreeImport> struTreeMap = this.buildStrutsTree(struChildrenTreeList);
        // 获取根节点
        PredictionStruTreeImport rootNode = struTreeMap.get(parentStruNode.getStruId());
        if (ObjectUtils.isNotEmpty(lastChildStruNode)) {
            updateLeftAndRightNo(rootNode, lastChildStruNode.getRightNo());
        } else {
            updateLeftAndRightNo(rootNode, parentStruNode.getLeftNo());
        }

        updateDeep(rootNode, parentStruNode.getDeep());
        updateSeriesNo(rootNode, parentStruNode.getSeriesNo());
        updateType(rootNode);

        // 有需要插入的数据
        if (!CollectionUtils.isEmpty(newStruList)) {
            // 通过TreeMap构建treeList
            List<PredictionStruTree> struTreeList = new ArrayList<>();
            buildStruTreeList(rootNode, struTreeList, newStruList);

            int nodeCnt = newStruList.size() * 2;
            // 2.更新树节点-父级节点类型
            if (RpConstant.TREENODE_TYPE_LEAF.equals(parentStruNode.getType())) {
                parentStruNode.setType(RpConstant.TREENODE_TYPE_NONLEAF);
                parentStruNode.setUpdateUserBy(sysUser.getUsername());
                parentStruNode.setUpdateTime(new Date());
                predictionStruTreeMapper.updateById(parentStruNode);
            }

            // 计算偏移量
            // 3.更新树节点-左右序号维护
            predictionStruTreeMapper.updateLeftNoByChangeNum(rightNo, nodeCnt, predictionId);
            predictionStruTreeMapper.updateRightNoByChangeNum(rightNo, nodeCnt, predictionId);

            // 依次插入已读入的数据
            // 插入结构树数据
            this.saveStruTree(struTreeList, sysUser.getUsername());
        }
    }

    private PredictionStruTree buildStruTree(String rpId, String struId, String pid, String name) {
        PredictionStruTree struTree = new PredictionStruTree();
        struTree.setPredictionId(rpId);
        struTree.setStruId(struId);
        struTree.setParentStruId(pid);
        struTree.setName(name);
        struTree.setStruTypeCode(RpConstant.STRU_TYPE_STRU);
        return struTree;
    }

    private Map<String, PredictionStruTreeImport> buildStrutsTree(List<PredictionStruTree> treeList) {
        Map<String, PredictionStruTreeImport> importStruTreeMap = Maps.newTreeMap();
        // 将所有节点先放入map
        for (PredictionStruTree struTree : treeList) {
            String struId = struTree.getStruId();
            PredictionStruTreeImport importStruTree = new PredictionStruTreeImport();
            importStruTree.setPredictionStruTree(struTree);
            importStruTreeMap.put(struId, importStruTree);
        }
        // 根据上面放进去的节点，将所有节点分父子关系存入map
        for (PredictionStruTree struTree : treeList) {
            String struId = struTree.getStruId();
            String parentStruId = struTree.getParentStruId();
            PredictionStruTreeImport importStruTree = importStruTreeMap.get(parentStruId);
            if (importStruTree != null) {
                List<PredictionStruTreeImport> children = importStruTree.getChildren();
                if (CollectionUtils.isEmpty(children)) {
                    children = new ArrayList<>();
                    PredictionStruTreeImport importStruTree1 = importStruTreeMap.get(struId);
                    children.add(importStruTree1);
                } else {
                    PredictionStruTreeImport importStruTree1 = importStruTreeMap.get(struId);
                    children.add(importStruTree1);
                }
                importStruTree.setChildren(children);
            }
        }
        return importStruTreeMap;
    }

    private Integer updateLeftAndRightNo(PredictionStruTreeImport importStruTree, Integer leftNo) {
        PredictionStruTree struTree = importStruTree.getPredictionStruTree();
        List<PredictionStruTreeImport> children = importStruTree.getChildren();

        Integer lastNo = leftNo;

        struTree.setLeftNo(leftNo);
        if (!CollectionUtils.isEmpty(children)) {
            // 结构
            for (PredictionStruTreeImport child : children) {
                String struTypeCode = child.getPredictionStruTree().getStruTypeCode();
                if (RpConstant.STRU_TYPE_STRU.equals(struTypeCode)) {
                    lastNo = updateLeftAndRightNo(child, lastNo + 1);
                }
            }
        }
        struTree.setRightNo(lastNo + 1);
        return struTree.getRightNo();
    }

    private void updateDeep(PredictionStruTreeImport importStruTree, Integer deep) {
        PredictionStruTree struTree = importStruTree.getPredictionStruTree();
        List<PredictionStruTreeImport> children = importStruTree.getChildren();
        struTree.setDeep(deep);
        if (!CollectionUtils.isEmpty(children)) {
            for (PredictionStruTreeImport child : children) {
                updateDeep(child, deep + 1);
            }
        }
    }

    private void updateSeriesNo(PredictionStruTreeImport importStruTree, String seriesNo) {
        PredictionStruTree struTree = importStruTree.getPredictionStruTree();
        List<PredictionStruTreeImport> children = importStruTree.getChildren();
        struTree.setSeriesNo(seriesNo);
        if (!CollectionUtils.isEmpty(children)) {
            // 结构
            int struIndex = 1;
            for (PredictionStruTreeImport child : children) {
                String struTypeCode = child.getPredictionStruTree().getStruTypeCode();
                if (RpConstant.STRU_TYPE_STRU.equals(struTypeCode)) {
                    String childSeriesNo;
                    if (RpConstant.ROOTNODE_PARENT.equals(child.getPredictionStruTree().getParentStruId())) {
                        childSeriesNo = "1";
                    } else {
                        childSeriesNo = seriesNo + "." + struIndex;
                    }
                    struIndex++;
                    updateSeriesNo(child, childSeriesNo);
                }
            }
        }
    }

    private void updateType(PredictionStruTreeImport importStruTree) {
        PredictionStruTree struTree = importStruTree.getPredictionStruTree();
        List<PredictionStruTreeImport> children = importStruTree.getChildren();
        struTree.setType(RpConstant.TREENODE_TYPE_NONLEAF);
        if (CollectionUtils.isEmpty(children)) {
            struTree.setType(RpConstant.TREENODE_TYPE_LEAF);
            return;
        }
        for (PredictionStruTreeImport child : children) {
            updateType(child);
        }
    }

    private void buildStruTreeList(PredictionStruTreeImport predictionStruTreeImport, List<PredictionStruTree> struTreeList, List<String> newStruList) {
        List<PredictionStruTreeImport> children = predictionStruTreeImport.getChildren();
        if (newStruList.contains(predictionStruTreeImport.getPredictionStruTree().getStruId())) {
            struTreeList.add(predictionStruTreeImport.getPredictionStruTree());
        }
        if (!CollectionUtils.isEmpty(children)) {
            for (PredictionStruTreeImport child : children) {
                buildStruTreeList(child, struTreeList, newStruList);
            }
        }
    }

    /**
     * 插入结构树表
     * @param struTreeList
     */
    private void saveStruTree(List<PredictionStruTree> struTreeList, String userName) {
        List<PredictionStruTree> batchStruTrees = new ArrayList<>();
        for (PredictionStruTree struTree: struTreeList) {
            struTree.setInsertUserBy(userName);
            struTree.setInsertTime(new Date());
            batchStruTrees.add(struTree);
        }
        this.saveBatch(batchStruTrees);
    }
}
